@@ -1,13 +1,13 @@
 """
-Plotapi - Engaging visualisations, made easy.
+PlotAPI - Engaging visualisations, made easy.
 
 This package enables the generation of beautiful visualisations. 
 They can be saved directly to HTML, PNG, PDF, and MP4 files.
 They can also be displayed in a Jupyter Notebook output cell.
 
-Copyright 2021-2022, Dr. Shahin Rostami
-http://shahinrostami.com
-http://plotapi.com
+Copyright 2021-2023, Dr. Shahin Rostami
+https://shahinrostami.com
+https://plotapi.com
 https://github.com/shahinrostami/plotapi
 https://pypi.org/project/plotapi/
 """
@@ -16,11 +16,12 @@ MIT
 """
 import json
 import os
+
 import requests
 
 session = requests.Session()
 url = "https://plotapi.com"
-ssl_verficiation = True
+ssl_verification = True
 path = os.path.dirname(__file__)
 
 try:
@@ -46,7 +47,7 @@ def api_key(key):
             print(key, file=api_key_file)
 
         print(
-            "Your Plotapi API key has been saved in your local environment. You will not need to set it again."
+            "Your PlotAPI API key has been saved in your local environment. You will not need to set it again."
         )
     except:
         pass
@@ -57,8 +58,8 @@ class Visualisation:
         api_key(key)
 
     def verify_ssl(truth):
-        global ssl_verficiation
-        ssl_verficiation = truth
+        global ssl_verification
+        ssl_verification = truth
 
     def set_license(username, password):
         global session
@@ -68,6 +69,10 @@ class Visualisation:
         self.params = params
         self.endpoint = endpoint
 
+    def _ipython_display_(self):
+        """Display the visualisation (as a side effect) within a notebook"""
+        self.show()
+
     """PlotAPI Generation"""
 
     def get_html(self):
@@ -77,7 +82,7 @@ class Visualisation:
             f"{url}/{self.endpoint}/",
             json=params,
             headers={"Content-Encoding": directive},
-            verify=ssl_verficiation,
+            verify=ssl_verification,
         )
 
         if result.status_code == 200:
@@ -86,14 +91,15 @@ class Visualisation:
             detail = json.loads(result.content.decode("utf8"))
             raise Exception(detail)
 
-    def get_png(self):
+    def get_png(self, query_params=None):
         """Generates the PNG using the Plotapi service."""
         params, directive = content_encoding(self.params)
         result = session.post(
             f"{url}/{self.endpoint}/png",
             json=params,
+            params=query_params,
             headers={"Content-Encoding": directive},
-            verify=ssl_verficiation,
+            verify=ssl_verification,
         )
 
         if result.status_code == 200:
@@ -102,14 +108,14 @@ class Visualisation:
             detail = json.loads(result.content.decode("utf8"))
             raise Exception(detail)
 
-    def get_pdf(self):
+    def get_pdf(self, query_params=None):
         """Generates the PDF using the Plotapi service."""
         params, directive = content_encoding(self.params)
         result = session.post(
             f"{url}/{self.endpoint}/pdf",
             json=params,
             headers={"Content-Encoding": directive},
-            verify=ssl_verficiation,
+            verify=ssl_verification,
         )
 
         if result.status_code == 200:
@@ -118,14 +124,14 @@ class Visualisation:
             detail = json.loads(result.content.decode("utf8"))
             raise Exception(detail)
 
-    def get_mp4(self):
+    def get_mp4(self, query_params=None):
         """Generates the MP4 using the Plotapi service."""
         params, directive = content_encoding(self.params)
         result = session.post(
             f"{url}/{self.endpoint}/mp4",
             json=params,
             headers={"Content-Encoding": directive},
-            verify=ssl_verficiation,
+            verify=ssl_verification,
         )
 
         if result.status_code == 200:
@@ -134,12 +140,46 @@ class Visualisation:
             detail = json.loads(result.content.decode("utf8"))
             raise Exception(detail)
 
-    def to_string(self, filename="out.html"):
+    def to_string(self):
         """Outputs the generated HTML to as a string."""
         html = self.get_html()
         return html
 
-    """ File output"""
+    """Upload and share"""
+
+    def upload(
+        self,
+        name="My First Upload",
+        public=False,
+        description="Uploaded from the API",
+        custom_css="",
+        maximized=False,
+        full_width=False,
+    ):
+        """Upload visualization to PlotAPI.com."""
+        self.params["plotapi_name"] = name
+        self.params["plotapi_public"] = public
+        self.params["plotapi_description"] = description
+        self.params["plotapi_custom_css"] = custom_css
+        self.params["plotapi_maximized"] = maximized
+        self.params["plotapi_full_width"] = full_width
+
+        params, directive = content_encoding(self.params)
+        result = session.post(
+            f"{url}/upload/{self.endpoint}",
+            json=params,
+            headers={"Content-Encoding": directive},
+            verify=ssl_verification,
+        )
+
+        detail = json.loads(result.content.decode("utf8"))
+
+        if result.status_code == 200:
+            return detail
+        else:
+            raise Exception(detail)
+
+    """File output"""
 
     def to_html(self, filename="out.html"):
         """Outputs the generated HTML to a HTML file."""
@@ -148,47 +188,47 @@ class Visualisation:
         file.write(html)
         file.close()
 
-    def to_png(self, filename="out.png"):
+    def to_png(self, filename="out.png", **kwargs):
         """Outputs the generated PNG to a file."""
-        image = self.get_png()
+        image = self.get_png(kwargs)
         file = open(filename, "wb")
         file.write(image)
         file.close()
 
-    def to_pdf(self, filename="out.pdf"):
+    def to_pdf(self, filename="out.pdf", **kwargs):
         """Outputs the generated PDF to a file."""
         pdf = self.get_pdf()
         file = open(filename, "wb")
         file.write(pdf)
         file.close()
 
-    def to_mp4(self, filename="out.mp4"):
+    def to_mp4(self, filename="out.mp4", **kwargs):
         """Outputs the generated MP4 to a file."""
         pdf = self.get_mp4()
         file = open(filename, "wb")
         file.write(pdf)
         file.close()
 
-    """ Jupyter Lab output"""
+    """Jupyter Lab output"""
 
     def show(self):
         """Outputs the generated HTML to a Jupyter Lab output cell."""
-        from IPython.display import display, HTML
+        from IPython.display import HTML, display
 
         html = self.get_html()
         display(HTML(html))
 
-    def show_png(self):
+    def show_png(self, **kwargs):
         """Outputs the generated PNG to a Jupyter Lab output cell."""
-        from IPython.display import display, Image
+        from IPython.display import Image, display
 
-        image = self.get_png()
+        image = self.get_png(kwargs)
         display(Image(image))
 
-    def show_mp4(self):
+    def show_mp4(self, **kwargs):
         """Outputs the generated MP4 to a Jupyter Lab output cell."""
-        from IPython.display import display
         from IPython.core.display import Video
+        from IPython.display import display
 
         video = self.get_mp4()
         display(
@@ -264,3 +304,12 @@ class ParetoFront(Visualisation):
         params["samples"] = samples
         endpoint = "paretofront"
         super().__init__(params, endpoint)
+
+
+class SplitChord(Visualisation):
+    def __init__(self, links=None, nodes=None, **kwargs):
+        params = kwargs
+        params["links"] = links
+        params["nodes"] = nodes
+        endpoint = "splitchord"
+        super().__init__(params, endpoint)
